#version 150
precision highp float; precision highp int;
//Ethan Alexander Shulman 2023 "Alien Signals from Space" June 17, 2023

in vec3 in_Position;
out vec4 uv;
out float time;

layout(std140) uniform ShaderviewBlock {
	vec4 mouse;
	float screenX,screenY;
	int frame,framerate;
};
layout(std140) uniform MidiBlock {
	vec4 midi[64];//x = note id, y = velocity, z = channel/track, w = time in seconds since note pressed
	int midiCount;
};
out vec4 music;//x = lead, y = snare, z = kick, w = charlatan
out float orph;//orpheus
/*LMMS midi converter https://www.lynxwave.com/LMMStoMIDI/LMMStoMIDIConverter.html since LMMS midi export is broken.

Launch options for midi config 'Shaderview console AlienSignalsFromSpace.mid 6 0.4'
6 is the bitmask 0b00110 signifying which instruments are beats/drums with a fixed length.
1 is the fixed beat length in seconds.

Midi Instrument 0 = GenericLead
Midi Instrument 1 = snare_muffled01.ogg
Midi Instrument 2 = kick_hard01.ogg
Midi Instrument 3 = Orpheus
Midi Instrument 4 = Charlatan*/

#define TEMPO_TIME(bpm,note) (60.*4.*note/bpm)


#define AA_SAMPLES 5
#define ANIMATION 1

#define PI 3.141592653589793


mat2 r2d(float a) {
	float c = cos(a), s = sin(a);
	return mat2(c,s,-s,c);
}
//r2 hash http://extremelearning.com.au/unreasonable-effectiveness-of-quasirandom-sequences/
vec2 quasiRandom(float n) {
	#define R2V 1.32471795724474602596
	return fract(.5+n*(1./vec2(R2V,R2V*R2V)));
}

float triwave(float x) {
	return abs(fract(x)*2.-1.);
}

vec3 hue(float h) {
	return clamp(abs(mod(h*6.+vec3(0.,4.,2.),6.)-3.)-1.,0.,1.);
}


float stars(vec2 p, vec2 deformFreq, float deform, float radius, float sharpness) {
	return clamp(1.-(length(fract(p+cos(p.yx*deformFreq)*deform)-.5)-radius)*sharpness, 0.,1.);
}


float voronoiCell(vec2 p, vec2 deformFreq, float deform, float cellSize) {
	return length(mod(p+cos(p.yx*deformFreq)*deform,cellSize)-cellSize*.5);
}

float voronoi(vec2 p, vec2 deformFreq, float deform, float cellSize, int iter) {
	float d = 1e8;
	for (int i = 0; i < iter; i++) {
		d = min(d, voronoiCell(p+quasiRandom(float(i))*cellSize, deformFreq,deform, cellSize));
	}
	return d;
}


uniform sampler2D Buffer0;

void vert() {
	#ifdef ANIMATION
	time = float(frame)/float(framerate*AA_SAMPLES);
	
	music = vec4(0);
	for (int i = 0; i < midiCount; i++) {
		switch (int(midi[i].z)) {
			case 0://lead
				music.x += .5+.5*max(0.,1.-midi[i].w/.2);
				break;
				
			case 1://snare
				music.y += max(0.,1.-midi[i].w/.1);
				break;
			
			case 2://kick
				music.z += pow(max(0.,1.-midi[i].w/.25),.5);
				break;
				
			case 3://orpheus
				orph = cos(midi[i].w*(PI*2.)/TEMPO_TIME(160.,1./2.)-PI)*.5+.5;
				break;
				
			case 4://charlatan
				if (midi[i].w < .75) music.w += max(0., 1.-fract(midi[i].w/TEMPO_TIME(160.,1./8.))*2.2);
				break;
		}
	}
	
	
	#else
	
	//screenshot 1
	time = 0.;
	music = vec4(1,0,0,0);
	orph = 0.4;
	
	//screenshot 2
	time = 100.;
	music = vec4(1);
	orph = .5;
	#endif
	
	uv = vec4((in_Position.xy*vec2(screenX/screenY,1)+(quasiRandom(float(frame%AA_SAMPLES))-.5)*2./screenY),
		(in_Position.xy*.5+.5)*vec2(screenX,screenY));
	gl_Position = vec4(in_Position,1);
}
void frag() {
	//star scape
	vec3 ss = vec3(0);
	for (float l = 0.; l < 8.1; l++) {
		ss += mix(hue(l/12.),vec3(1),.35)*stars((uv.xy+quasiRandom(l))*(15.-l), vec2(.7)*r2d(l*.378621),1., .01,70.);
	}
	ss *= .02+1.8*pow(voronoi(uv.xy*.6, vec2(1.4,1.7),.3, 1.,4), 2.);
	
	//nebula
	vec3 nb = (hue(.65)*.6*pow(voronoi(uv.xy*.34+.4, vec2(.6,.3),.2, 1.,2),3.)+
		hue(.01)*.2*pow(voronoi(uv.xy*.73+.9732, vec2(.283,.4589),.18, 1.,3),3.))*
			(.2+.8*pow(voronoi(uv.xy*14.+time*vec2(.04,-.06), vec2(.9,.41),1., 1.,4)*3.,4.));
			
	//star
	vec3 star = vec3(0);
	float slen = length(uv.xy), ang = atan(uv.x,uv.y);
	#define star_add(color,atten,beams,beamSize, flare,flareDst,flareAtten,flareBeams,flareBeamSize) star += (color)*(1./(1.+pow(slen+(sin(ang*(beams))*.5+.5)*(beamSize),2.)*(atten))+\
(flare)/(1.+pow(abs(slen-(flareDst)+pow(triwave(ang*flareBeams/(PI*2.)),.5)*(flareBeamSize)),2.)*(flareAtten)))
	
	
	star_add(vec3(.45,.72,1.)*2.,15000.,6.,.004, .005,.5,100.,11.,.3);//main constant center
	
	star_add(vec3(1.,.5,0)*.5*music.x,4000.,12.,.01, .03,.68,50.,8.,.05);//lead
	
	star_add(vec3(.7,.3,1.)*music.y,5000.,8.,.04, .03,.6,300.,5.,.2);//snare
	
	star_add(vec3(.4,.2,1.)*2.*music.z,10000.,8.,.001, .04,.4,300.,6.,.1);//kick
	
	star_add(vec3(1,.2,.5)*music.w,1000.,12.,.01, 0.1,.3+music.w*.1,500.,4.,.25);//charlatan
	
	star_add(vec3(.1,.3,1)*2.*orph,500.,6.,.05, 0.04,0.3+orph*.5,50.,22.,0.1);
	
	
	//final composition
	vec3 c = ss+nb+star;
	
	vec4 sum = texelFetch(Buffer0,ivec2(uv.zw),0);
	if (frame%AA_SAMPLES == 0) sum = vec4(0);
	sum += vec4(c, 1.);
	gl_FragColor = sum;
}
